/*!
 * ---------------------------------------------------------------------
 *
 * GLPI - Gestionnaire Libre de Parc Informatique
 *
 * http://glpi-project.org
 *
 * @copyright 2015-2026 Teclib' and contributors.
 * @copyright 2003-2014 by the INDEPNET Development Team.
 * @licence   https://www.gnu.org/licenses/gpl-3.0.html
 *
 * ---------------------------------------------------------------------
 *
 * LICENSE
 *
 * This file is part of GLPI.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * ---------------------------------------------------------------------
 */

:root {
    --is-dark: #{$is-dark};
    --animate-duration: 0.3s;
    --light: #{$light};
    --dark: #{$dark};
    --logo: #{$logo};
    --contrast-light: #{$color-contrast-light};
    --contrast-dark: #{$color-contrast-dark};
    --light-mix: #{rgba(mix($light, $dark, 64%), 0.24)};
    --hover-bg: #{$hover-bg};
    --btn-disabled-opacity: #{$btn-disabled-opacity};

    // Dark mode
    --dark-mode-darken: #{$dark-mode-darken};
    --dark-mode-text: #{$dark-mode-text};

    // Form elements
    --input-bg: #{$input-bg};
    --input-color: #{$input-color};
    --input-border: #{$input-border};
    --input-border-color: #{$input-border-color};
    --input-border-radius: #{$border-radius};
    --input-border-width: #{$input-border-width};

    // headers
    --header-fg: #{$header-fg};
    --header-bg: #{$header-bg};
    --header-border-color: #{$header-border-color};

    // card
    --card-border-color: #{$card-border-color};

    // table
    --table-head-border-color: #{$table-head-border-color};
    --table-head-color: #{$table-head-color};
    --table-head-bg: #{$table-head-bg};
}
