/*!
 * ---------------------------------------------------------------------
 *
 * GLPI - Gestionnaire Libre de Parc Informatique
 *
 * http://glpi-project.org
 *
 * @copyright 2015-2026 Teclib' and contributors.
 * @copyright 2003-2014 by the INDEPNET Development Team.
 * @licence   https://www.gnu.org/licenses/gpl-3.0.html
 *
 * ---------------------------------------------------------------------
 *
 * LICENSE
 *
 * This file is part of GLPI.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * ---------------------------------------------------------------------
 */

$is-dark: true;
$primary: #b6c3e0;
$primary-fg: #232e3c;
$secondary: #b0bbd4;
$secondary-fg: #262d3d;

.btn-primary,
.btn-outline-primary,
.btn-ghost-primary {
    --tblr-btn-color-text: red;
}

$dark: #000;
$dark-mode-darken: #000;
$color-contrast-dark: #1a1a1a;
$light: #e6e6e6;
$link-color: #b6c3e0;
$text-muted: #c6cad2;
$mainmenu_bg: #2f3f64;
$mainmenu_fg: #f4f6fa;
$logo: "../pics/logos/logo-GLPI-100-white.png";
$logo_reduced: "../pics/logos/logo-G-100-white.png";
$badge-bg: #d0d9f0;
$badge-fg: rgb(58, 73, 102);
$header-fg: #6ba8f3;
$header-border-color: transparent;
$header-badge-bg: #1d2c3f;
$header-badge-fg: #6ba8f3;
$tabs-bg: $color-contrast-dark;
$tabs-fg: $light;
$tabs-border: transparent;
$tabs-active-bg: #737373;
$tabs-active-fg: #fff;
$tabs-active-border: transparent;
$badge-bg: $light;
$badge-fg: $dark;
$input-border: #555;
$input-border-color: #555;
$input-focus-bg: $dark;
$input-focus-border-color: #555;
$input-disabled-bg: #35404e;
$table-border-color: #1f2936;
$table-head-bg: #1f2936;
$table-head-color: $light;
$search_border: rgba(101, 109, 119, 16%);
$table_border: rgba(101, 109, 119, 16%);
$border-color: rgba(101, 109, 119, 16%);
$itil-secondary-bg: #1d2531 !default;
$timeline-fup-bg: #1c2430 !default;
$timeline-fup-fg: #8892a8 !default;
$timeline-fup-border: #2f3d50 !default;
$code-color: rgb(180, 180, 180);

@import "../includes/palette_dark";
