{#
 # ---------------------------------------------------------------------
 #
 # GLPI - Gestionnaire Libre de Parc Informatique
 #
 # http://glpi-project.org
 #
 # @copyright 2015-2026 Teclib' and contributors.
 # @licence   https://www.gnu.org/licenses/gpl-3.0.html
 #
 # ---------------------------------------------------------------------
 #
 # LICENSE
 #
 # This file is part of GLPI.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # ---------------------------------------------------------------------
 #}

{% import 'components/form/fields_macros.html.twig' as macros %}

<form action='{{ target }}' method="post">
    <div class="container">
        <div class="row justify-content-evenly">
            <div class="col-12 col-xxl-8">
                <div class="card">
                    <h3 class="card-header">
                            {{ __('Please choose LDAP directory to import users and groups from') }}
                    </h3>
                    {% if nb_ldap_servers > 1 %}
                        <div class="card-body">
                            {% set ldap_dropdown %}
                                {% do call('AuthLDAP::Dropdown', [{
                                    'name': 'ldap_server',
                                    'display_emptychoice': false,
                                    'comment': true,
                                    'condition': {'is_active': 1}
                                }]) %}
                            {% endset %}

                            {{ macros.field(
                                'ldap_server',
                                ldap_dropdown,
                                _n('LDAP directory', 'LDAP directories', 1),
                                {full_width: true}
                            ) }}
                        </div>
                        <div class="card-footer d-flex justify-content-end">
                            <button type="submit" class="btn btn-primary" name="ldap_showusers">
                                <i class="ti ti-arrow-right"></i>
                                <span>{{ _x('button', 'Post') }}</span>
                            </button>
                        </div>
                    {% else %}
                        <div class="card-body">
                            <div class="alert alert-warning" role="alert">
                                <h4 class="alert-title">{{ __('No LDAP directory defined in GLPI') }}</h4>
                            </div>
                        </div>
                    {% endif %}
                </div>
            </div>
        </div>
    </div>
    <input type="hidden" name="_glpi_csrf_token" value="{{ csrf_token() }}" />
</form>
