/*!
 * ---------------------------------------------------------------------
 *
 * GLPI - Gestionnaire Libre de Parc Informatique
 *
 * http://glpi-project.org
 *
 * @copyright 2015-2026 Teclib' and contributors.
 * @copyright 2003-2014 by the INDEPNET Development Team.
 * @licence   https://www.gnu.org/licenses/gpl-3.0.html
 *
 * ---------------------------------------------------------------------
 *
 * LICENSE
 *
 * This file is part of GLPI.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * ---------------------------------------------------------------------
 */

/**
Override the override of mixins by tabler over bootstrap
which broke styles of btn-group > .btn-check
*/

@mixin button-check(
    $color,
    $color-hover: color-contrast($color),
    $active-background: $color,
    $active-border: $color,
    $active-color: color-contrast($active-background)
) {
    .btn-check:checked + &,
    .btn-check:active + & {
        color: $active-color;
        background-color: $active-background;
        border-color: $active-border;
    }
}

@each $color, $value in $theme-colors {
    .btn-#{$color} {
        @include button-check($value, $value);
    }
}

@each $color, $value in $theme-colors {
    .btn-outline-#{$color} {
        @include button-check($value);
    }
}

.btn-group.flex-row-reverse > .btn {
    border-radius: 0 !important;
}

.btn-group.flex-row-reverse > .btn:first-child {
    border-top-right-radius: 4px !important;
    border-bottom-right-radius: 4px !important;
}

.btn-group.flex-row-reverse > .btn:last-child {
    border-top-left-radius: 4px !important;
    border-bottom-left-radius: 4px !important;
    border-right: unset !important;
}

.btn-group > .btn:not(:last-child):not(.dropdown-toggle), .btn-group > .btn-group:not(:last-child) > .btn {
    border-right: unset !important;
}
