<?php

namespace TrueBV\Exception;

/**
 * Class OutOfBoundsException
 * @package TrueBV\Exception
 * @author  Sebastian Kroczek <sk@xbug.de>
 */
class OutOfBoundsException extends \RuntimeException
{

}
