@import "~jsvectormap/dist/css/jsvectormap";

.jvm-tooltip {
  background: $dark;
  font-family: inherit;
  font-size: $h5-font-size;
  box-shadow: $card-shadow;
}

.jvm-series-container .jvm-legend  .jvm-legend-title {
  border-bottom: 0;
  font-weight: $font-weight-medium;
}

.jvm-series-container .jvm-legend {
  background-color: var(--#{$variable-prefix}card-bg);
  border: 1px solid var(--#{$variable-prefix}border-color);
  min-width: 8rem;

  .jvm-legend-inner .jvm-legend-tick {
    display: flex;
    align-items: center;

    .jvm-legend-tick-sample {
      width: .75rem;
      height: .75rem;
    }
  }
}

.jvm-zoom-btn {
  border: 1px solid var(--#{$variable-prefix}border-color);
  background: var(--#{$variable-prefix}card-bg);
  color: var(--#{$variable-prefix}body-color);
  padding: 0;
  display: flex;
  align-items: center;
  justify-content: center;
  line-height: 1;
  width: 1.5rem;
  height: 1.5rem;
  font-size: 1rem;
  box-shadow: $card-shadow;

  &.jvm-zoomout {
    top: 2.5rem;
  }
}