{#
 # ---------------------------------------------------------------------
 #
 # GLPI - Gestionnaire Libre de Parc Informatique
 #
 # http://glpi-project.org
 #
 # @copyright 2015-2026 Teclib' and contributors.
 # @licence   https://www.gnu.org/licenses/gpl-3.0.html
 #
 # ---------------------------------------------------------------------
 #
 # LICENSE
 #
 # This file is part of GLPI.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # ---------------------------------------------------------------------
 #}

{% set users_id = session('glpiID') %}

<div class="itil-timeline">
   <div class="timeline-item ITILContent">
      <div class="row">
         <div class="col-auto d-flex flex-column picture-block">
            {{ include('components/user/picture.html.twig', {'users_id': users_id}, with_context = false) }}
         </div>
         <div class="col-12 col-sm content-block">
            <div class="d-flex">
               <div class="d-none d-md-block">
                  <strong>{{ get_item_link('User', users_id) }}</strong>
               </div>
            </div>
            <span class="mt-2 timeline-content left card">
               <div class="card-body">
                  {% if item.isNewItem() %}
                     <div class="alert alert-info" role="alert">
                        {% set entitybadge %}
                           <span class="ms-1">
                              {{ call('Entity::badgeCompletenameById', [item.fields['entities_id']])|raw }}
                           </span>
                        {% endset %}

                        {{ __('%1$s will be added in entity %2$s')|format(
                           item.getTypeName(1),
                           entitybadge
                        )|raw }}
                     </div>
                  {% endif %}

                  {{ include('components/itilobject/timeline/simple_form.html.twig', {
                     'no_form': true
                  }) }}
               </div>
            </span>
         </div>
      </div>
   </div>
</div>
