{#
 # ---------------------------------------------------------------------
 #
 # GLPI - Gestionnaire Libre de Parc Informatique
 #
 # http://glpi-project.org
 #
 # @copyright 2015-2026 Teclib' and contributors.
 # @licence   https://www.gnu.org/licenses/gpl-3.0.html
 #
 # ---------------------------------------------------------------------
 #
 # LICENSE
 #
 # This file is part of GLPI.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # ---------------------------------------------------------------------
 #}

<div class="container">
   <div class="row justify-content-evenly">
      <div class="col-12 col-xxl-5">
         <div class="card">
            <div class="card-header">
               <h3>{{ __('Bulk import users from a LDAP directory') }}</h3>
            </div>
            <div class="list-group list-group-flush">
               <a class="list-group-item list-group-item-action" href="{{ path('front/ldap.import.php') }}?mode=1&amp;action=show">
                  <i class="fas fa-fw fa-users-cog me-1"></i>
                  <span>{{ __('Synchronizing already imported users') }}</span>
               </a>

               <a class="list-group-item list-group-item-action" href="{{ path('front/ldap.import.php') }}?mode=0&amp;action=show">
                  <i class="fas fa-fw fa-user-plus me-1"></i>
                  <span>{{ __('Import new users') }}</span>
               </a>
            </div>
         </div>
      </div>
   </div>
</div>
