/*!
 * ---------------------------------------------------------------------
 *
 * GLPI - Gestionnaire Libre de Parc Informatique
 *
 * http://glpi-project.org
 *
 * @copyright 2015-2026 Teclib' and contributors.
 * @copyright 2003-2014 by the INDEPNET Development Team.
 * @licence   https://www.gnu.org/licenses/gpl-3.0.html
 *
 * ---------------------------------------------------------------------
 *
 * LICENSE
 *
 * This file is part of GLPI.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * ---------------------------------------------------------------------
 */

.netport {
    &:target {
        animation: shakeX; // from animate css dependencies
        animation-duration: 2s;
        scroll-margin-top: 5em;
    }

    td {
        &.green {
            background-color: green;
            color: white;
        }

        &.red {
            background-color: red;
            color: white;
        }

        &.orange {
            background-color: orange;
            color: white;
        }
    }

    &.trunk {
        @if $is-dark {
            background-color: rgb(56, 110, 92);
            color: white;

            a {
                color: white;
            }
        } @else {
            background-color: mediumaquamarine;
        }
    }

    &.hub {
        @if $is-dark {
            background-color: rgb(185, 112, 88);
            color: white;

            a {
                color: white;
            }
        } @else {
            background-color: lightsalmon;
        }
    }

    &.cotrunk {
        @if $is-dark {
            background-color: rgb(77, 77, 77);
            color: white;

            a {
                color: white;
            }
        } @else {
            background-color: silver;
        }
    }

    &.aggregated,
    .aggregated {
        background-color: teal;
        color: white;
        padding-left: 2em;

        a {
            color: white;
        }
    }

    .hub {
        border: 1px grey solid;
        margin: 0 0.5em;

        div {
            padding: 0.2em 0.5em;
            border-bottom: 1px grey solid;

            &:last-of-type {
                border-bottom: none;
            }
        }
    }
}

.netport-legend {
    width: 100%;

    td.netport {
        text-align: center;
        width: 25%;
    }

    @if $is-dark {
        color: white;
    }
}
