{#
 # ---------------------------------------------------------------------
 #
 # GLPI - Gestionnaire Libre de Parc Informatique
 #
 # http://glpi-project.org
 #
 # @copyright 2015-2026 Teclib' and contributors.
 # @licence   https://www.gnu.org/licenses/gpl-3.0.html
 #
 # ---------------------------------------------------------------------
 #
 # LICENSE
 #
 # This file is part of GLPI.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # ---------------------------------------------------------------------
 #}

{# JS scripts / modules are loaded in the footer in non-anonymous pages #}
{% set js_files = js_files|merge(get_plugins_js_scripts_files(false)) %}
{% set js_modules = js_modules|merge(get_plugins_js_modules_files(false)) %}

            </main> {# #page #}
         </div> {# .page-body #}
      </div> {# .page-wrapper #}
   </div> {# .page #}

   <div class="floating-buttons d-inline-flex">
      <span class="btn btn-secondary d-none me-1" id="backtotop">
         <i class="fas fa-arrow-up" title="{{ __('Back to top of the page') }}">
            <span class="visually-hidden">{{ __('Top of the page') }}</span>
         </i>
      </span>
   </div>

   {% if config('maintenance_mode') %}
      <div id="maintenance-float">
         <a href="{{ path('front/config.form.php?forcetab=Config$5') }}">GLPI MAINTENANCE MODE</a>
      </div>
   {% endif %}

   {% for js_file in js_files %}
      <script type="text/javascript" src="{{ js_path(js_file.path, js_file.options ?? []) }}"></script>
   {% endfor %}

   {% for js_file in js_modules %}
      <script type="module" src="{{ js_path(js_file.path, js_file.options ?? []) }}"></script>
   {% endfor %}

</body>
</html>
