tinymce.addI18n("ca", {
    "Redo": "Refer",
    "Undo": "Desfer",
    "Cut": "Retalla",
    "Copy": "Copia",
    "Paste": "Enganxa",
    "Select all": "Seleccionar-ho tot",
    "New document": "Nou document",
    "Ok": "Acceptar",
    "Cancel": "Cancel\xb7la",
    "Visual aids": "Assist\xe8ncia visual",
    "Bold": "Negreta",
    "Italic": "Cursiva",
    "Underline": "Subratllat",
    "Strikethrough": "Barrat",
    "Superscript": "Super\xedndex",
    "Subscript": "Sub\xedndex",
    "Clear formatting": "Eliminar format",
    "Remove": "Eliminar",
    "Align left": "Alinea a l'esquerra",
    "Align center": "Alinea al centre",
    "Align right": "Alinea a la dreta",
    "No alignment": "Sense alineament",
    "Justify": "Justificat",
    "Bullet list": "Llista no ordenada",
    "Numbered list": "Llista enumerada",
    "Decrease indent": "Disminuir sagnat",
    "Increase indent": "Augmentar sagnat",
    "Close": "Tancar",
    "Formats": "Formats",
    "Your browser doesn't support direct access to the clipboard. Please use the Ctrl+X/C/V keyboard shortcuts instead.": "El vostre navegador no suporta l'acc\xe9s directe al portaobjectes. Si us plau, feu servir les dreceres de teclat Ctrl+X/C/V.",
    "Headings": "Encap\xe7alaments",
    "Heading 1": "Encap\xe7alament 1",
    "Heading 2": "Encap\xe7alament 2",
    "Heading 3": "Encap\xe7alament 3",
    "Heading 4": "Encap\xe7alament 4",
    "Heading 5": "Encap\xe7alament 5",
    "Heading 6": "Encap\xe7alament 6",
    "Preformatted": "Preformatat",
    "Div": "Div",
    "Pre": "",
    "Code": "Codi",
    "Paragraph": "Par\xe0graf",
    "Blockquote": "Cita",
    "Inline": "En l\xednia",
    "Blocks": "Blocs",
    "Paste is now in plain text mode. Contents will now be pasted as plain text until you toggle this option off.": "Enganxar ara \xe9s en mode text pla. Els continguts s'enganxaran com a text pla fins que desactivis aquesta opci\xf3.",
    "Fonts": "Fonts",
    "Font sizes": "Tamanys de font",
    "Class": "Classe",
    "Browse for an image": "Explorar per cercar una imatge",
    "OR": "O",
    "Drop an image here": "Deixar anar una imatge aqu\xed",
    "Upload": "Pujar",
    "Uploading image": "Pujant imatge",
    "Block": "Bloc",
    "Align": "Alinea",
    "Default": "Predeterminat",
    "Circle": "Cercle",
    "Disc": "Disc",
    "Square": "Quadrat",
    "Lower Alpha": "Lletra min\xfascula",
    "Lower Greek": "Lletra grega min\xfascula",
    "Lower Roman": "N\xfameros romans en min\xfascula",
    "Upper Alpha": "Lletra maj\xfascula",
    "Upper Roman": "N\xfameros romans en maj\xfascula",
    "Anchor...": "Ancoratge...",
    "Anchor": "\xc0ncora",
    "Name": "Nom",
    "ID": "ID",
    "ID should start with a letter, followed only by letters, numbers, dashes, dots, colons or underscores.": "La ID ha de comen\xe7ar amb una lletra, seguida nom\xe9s per lletres, nombres, guions, punts, dos punts o guions baixos.",
    "You have unsaved changes are you sure you want to navigate away?": "Teniu canvis sense desar, esteu segur que voleu deixar-ho ara?",
    "Restore last draft": "Restaurar l'\xfaltim esborrany",
    "Special character...": "Car\xe0cters especials\u2026",
    "Special Character": "Car\xe0cter especial",
    "Source code": "Codi font",
    "Insert/Edit code sample": "Inserir/editar mostra de codi",
    "Language": "Idioma",
    "Code sample...": "Mostra de codi...",
    "Left to right": "D'esquerra a dreta",
    "Right to left": "De dreta a esquerra",
    "Title": "T\xedtol",
    "Fullscreen": "Pantalla completa",
    "Action": "Acci\xf3",
    "Shortcut": "Acc\xe9s directe",
    "Help": "Ajuda",
    "Address": "Adre\xe7a",
    "Focus to menubar": "Enfocar la barra de men\xfa",
    "Focus to toolbar": "Enfocar la barra d'eines",
    "Focus to element path": "Enfocar la ruta d'elements",
    "Focus to contextual toolbar": "Enfocar la barra d'eines contextual",
    "Insert link (if link plugin activated)": "Inserir enlla\xe7 (si el complement d'enlla\xe7 est\xe0 activat)",
    "Save (if save plugin activated)": "Desar (si el complement desar est\xe0 activat)",
    "Find (if searchreplace plugin activated)": "Cercar (si el complement cercar-reempla\xe7ar est\xe0 activat)",
    "Plugins installed ({0}):": "Complements instal\xb7lats ({0}):",
    "Premium plugins:": "Complements pr\xe8mium:",
    "Learn more...": "Apr\xe8n m\xe9s...",
    "You are using {0}": "Est\xe0s utilitzant {0}",
    "Plugins": "Complements",
    "Handy Shortcuts": "Dreceres \xfatils",
    "Horizontal line": "L\xednia horitzontal",
    "Insert/edit image": "Insereix/edita imatge",
    "Alternative description": "Descripci\xf3 alternativa",
    "Accessibility": "Accessibilitat",
    "Image is decorative": "La imatge \xe9s decorativa",
    "Source": "Font",
    "Dimensions": "Dimensions",
    "Constrain proportions": "Conservar proporcions",
    "General": "Generals",
    "Advanced": "Avan\xe7ades",
    "Style": "Estil",
    "Vertical space": "Espai vertical",
    "Horizontal space": "Espai horitzontal",
    "Border": "Vora",
    "Insert image": "Inserir imatge",
    "Image...": "Imatge...",
    "Image list": "Llista d'imatges",
    "Resize": "Canviar mida",
    "Insert date/time": "Inserir data/hora",
    "Date/time": "Data/hora",
    "Insert/edit link": "Inserir/editar l\u2019enlla\xe7",
    "Text to display": "Text per visualitzar",
    "Url": "URL",
    "Open link in...": "Obrir l'enlla\xe7 a...",
    "Current window": "Finestra actual",
    "None": "Cap",
    "New window": "Finestra nova",
    "Open link": "Obrir l'enlla\xe7",
    "Remove link": "Treure l\u2019enlla\xe7",
    "Anchors": "\xc0ncores",
    "Link...": "Enlla\xe7...",
    "Paste or type a link": "Enganxa o escriu un enlla\xe7",
    "The URL you entered seems to be an email address. Do you want to add the required mailto: prefix?": "L'URL que has escrit sembla una adre\xe7a de correu electr\xf2nic. Vols afegir-li el prefix obligatori \xabmailto:\xbb?",
    "The URL you entered seems to be an external link. Do you want to add the required http:// prefix?": "L'URL que has escrit sembla un enlla\xe7 extern. Vols afegir-li el prefix obligatori \xabhttp://\xbb?",
    "The URL you entered seems to be an external link. Do you want to add the required https:// prefix?": "Sembla que l\u2019URL que has introdu\xeft \xe9s un enlla\xe7 extern. Vols afegir el prefix https:// necessari?",
    "Link list": "Llista d'enlla\xe7os",
    "Insert video": "Inserir v\xeddeo",
    "Insert/edit video": "Inserir/editar v\xeddeo",
    "Insert/edit media": "Inserir/editar multim\xe8dia",
    "Alternative source": "Font alternativa",
    "Alternative source URL": "URL de font alternativa",
    "Media poster (Image URL)": "Cartell de multim\xe8dia (URL d'imatge)",
    "Paste your embed code below:": "Enganxeu el codi a sota:",
    "Embed": "Incloure",
    "Media...": "Multim\xe8dia...",
    "Nonbreaking space": "Espai cont\xednu",
    "Page break": "Salt de p\xe0gina",
    "Paste as text": "Enganxar com a text",
    "Preview": "Visualitzaci\xf3 pr\xe8via",
    "Print": "Imprimir",
    "Print...": "Imprimir...",
    "Save": "Desar",
    "Find": "Cerca",
    "Replace with": "Reempla\xe7a per",
    "Replace": "Reempla\xe7ar",
    "Replace all": "Reempla\xe7a totes",
    "Previous": "Anterior",
    "Next": "Seg\xfcent",
    "Find and Replace": "Cercar i reempla\xe7ar",
    "Find and replace...": "Cercar i reempla\xe7ar...",
    "Could not find the specified string.": "No es pot trobar el text especificat.",
    "Match case": "Fes coincidir maj\xfascules i min\xfascules",
    "Find whole words only": "Cercar nom\xe9s paraules completes",
    "Find in selection": "Buscar a la selecci\xf3",
    "Insert table": "Inserir taula",
    "Table properties": "Propietats de taula",
    "Delete table": "Suprimir taula",
    "Cell": "Cel\xb7la",
    "Row": "Fila",
    "Column": "Columna",
    "Cell properties": "Propietats de cel\xb7la",
    "Merge cells": "Fusionar cel\xb7les",
    "Split cell": "Dividir cel\xb7les",
    "Insert row before": "Inserir fila a sobre",
    "Insert row after": "Inserir fila a sota",
    "Delete row": "Suprimir la fila",
    "Row properties": "Propietats de la fila",
    "Cut row": "Retallar la fila",
    "Cut column": "Retallar columna",
    "Copy row": "Copiar la fila",
    "Copy column": "Copiar columna",
    "Paste row before": "Enganxar fila a sobre",
    "Paste column before": "Enganxar columna abans",
    "Paste row after": "Enganxar fila a sota",
    "Paste column after": "Enganxar columna despr\xe9s",
    "Insert column before": "Inserir columna abans",
    "Insert column after": "Inserir columna despr\xe9s",
    "Delete column": "Suprimir columna",
    "Cols": "Columnes",
    "Rows": "Files",
    "Width": "Amplada",
    "Height": "Al\xe7ada",
    "Cell spacing": "Espai entre cel\xb7les",
    "Cell padding": "Marge intern",
    "Row clipboard actions": "Accions de fila del porta-retalls",
    "Column clipboard actions": "Accions de columna del porta-retalls",
    "Table styles": "Estils de taula",
    "Cell styles": "Estils de cel\xb7la",
    "Column header": "Cap\xe7alera de columna",
    "Row header": "Cap\xe7alera de fila",
    "Table caption": "T\xedtol de taula",
    "Caption": "Encap\xe7alament",
    "Show caption": "Mostrar encap\xe7alament",
    "Left": "Esquerra",
    "Center": "Centre",
    "Right": "Dreta",
    "Cell type": "Tipus de cel\xb7la",
    "Scope": "Abast",
    "Alignment": "Alineaci\xf3",
    "Horizontal align": "Alineaci\xf3 horitzontal",
    "Vertical align": "Alineaci\xf3 vertical",
    "Top": "Part superior",
    "Middle": "Centre",
    "Bottom": "Part inferior",
    "Header cell": "Cel\xb7la de cap\xe7alera",
    "Row group": "Grup de fila",
    "Column group": "Grup de columnes",
    "Row type": "Tipus de fila",
    "Header": "Encap\xe7alament",
    "Body": "Cos",
    "Footer": "Peu de p\xe0gina",
    "Border color": "Color de la vora",
    "Solid": "S\xf2lid",
    "Dotted": "Puntejat",
    "Dashed": "Guions",
    "Double": "Doble",
    "Groove": "Solc",
    "Ridge": "Carena",
    "Inset": "Insert",
    "Outset": "Relleu",
    "Hidden": "Ocult",
    "Insert template...": "Inserir plantilla...",
    "Templates": "Plantilles",
    "Template": "Plantilla",
    "Insert Template": "Inserir plantilla",
    "Text color": "Color del text",
    "Background color": "Color de fons",
    "Custom...": "Personalitza...",
    "Custom color": "Personalitzar el color",
    "No color": "Sense color",
    "Remove color": "Eliminar el color",
    "Show blocks": "Mostrar blocs",
    "Show invisible characters": "Mostrar car\xe0cters invisibles",
    "Word count": "Recompte de paraules",
    "Count": "Compta",
    "Document": "Document",
    "Selection": "Selecci\xf3",
    "Words": "Paraules",
    "Words: {0}": "Paraules: {0}",
    "{0} words": "{0} paraules",
    "File": "Arxiu",
    "Edit": "Editar",
    "Insert": "Inserir",
    "View": "Veure",
    "Format": "Format",
    "Table": "Taula",
    "Tools": "Eines",
    "Powered by {0}": "Desenvolupat per {0}",
    "Rich Text Area. Press ALT-F9 for menu. Press ALT-F10 for toolbar. Press ALT-0 for help": "\xc0rea de text amb format. Premeu ALT-F9 per mostrar el men\xfa, ALT F10 per la barra d'eines i ALT-0 per ajuda.",
    "Image title": "T\xedtol de la imatge",
    "Border width": "Amplada de la vora",
    "Border style": "Estil de la vora",
    "Error": "Error",
    "Warn": "Alerta",
    "Valid": "V\xe0lid",
    "To open the popup, press Shift+Enter": "Per obrir la finestra emergent, premeu Maj.+Retorn",
    "Rich Text Area": "\xc0rea de text enriquit",
    "Rich Text Area. Press ALT-0 for help.": "\xc0rea de Text enriquit. Premeu ALT-0 per obtenir ajuda.",
    "System Font": "Font del sistema",
    "Failed to upload image: {0}": "No s'ha pogut carregar la imatge: {0}",
    "Failed to load plugin: {0} from url {1}": "No s'ha pogut carregar el complement: {0} de l\u2019URL {1}",
    "Failed to load plugin url: {0}": "No s'ha pogut carregar l\u2019URL del complement: {0}",
    "Failed to initialize plugin: {0}": "No s'ha pogut inicialitzar el complement: {0}",
    "example": "exemple",
    "Search": "Cerca",
    "All": "Tot",
    "Currency": "Moneda",
    "Text": "Text",
    "Quotations": "Cites",
    "Mathematical": "S\xedmbols matem\xe0tics",
    "Extended Latin": "Llat\xed ampliat",
    "Symbols": "S\xedmbols",
    "Arrows": "Fletxes",
    "User Defined": "Definit per l'usuari",
    "dollar sign": "signe del d\xf2lar",
    "currency sign": "signe de la moneda",
    "euro-currency sign": "signe de l'euro",
    "colon sign": "signe del col\xf3n",
    "cruzeiro sign": "signe del cruzeiro",
    "french franc sign": "signe del franc franc\xe8s",
    "lira sign": "signe de la lira",
    "mill sign": "signe del mill",
    "naira sign": "signe de la naira",
    "peseta sign": "signe de la pesseta",
    "rupee sign": "signe de la rupia",
    "won sign": "signe del won",
    "new sheqel sign": "signe del nou x\xe9quel",
    "dong sign": "signe del dong",
    "kip sign": "signe del kip",
    "tugrik sign": "signe del t\xf6gr\xf6g",
    "drachma sign": "signe del dracma",
    "german penny symbol": "signe del penic alemany",
    "peso sign": "signe del peso",
    "guarani sign": "signe del guaran\xed",
    "austral sign": "signe de l\u2019austral",
    "hryvnia sign": "signe de la hr\xedvnia",
    "cedi sign": "signe del cedi",
    "livre tournois sign": "signe de la lliura tornesa",
    "spesmilo sign": "signe de l\u2019spesmilo",
    "tenge sign": "signe del tenge",
    "indian rupee sign": "signe de la rupia \xedndia",
    "turkish lira sign": "signe de la lira turca",
    "nordic mark sign": "signe del marc n\xf2rdic",
    "manat sign": "signe del manat",
    "ruble sign": "signe del ruble",
    "yen character": "signe del ien",
    "yuan character": "signe del iuan",
    "yuan character, in hong kong and taiwan": "signe del iuan en Hong Kong i Taiwan",
    "yen/yuan character variant one": "variaci\xf3 1 del signe del ien/iuan",
    "Emojis": "Emojis",
    "Emojis...": "Emojis...",
    "Loading emojis...": "Carregant emojis...",
    "Could not load emojis": "No s'han pogut carregar els emojis",
    "People": "Gent",
    "Animals and Nature": "Animals i natura",
    "Food and Drink": "Menjar i beure",
    "Activity": "Activitat",
    "Travel and Places": "Viatges i llocs",
    "Objects": "Objectes",
    "Flags": "Banderes",
    "Characters": "Car\xe0cters",
    "Characters (no spaces)": "Car\xe0cters (sense espais)",
    "{0} characters": "{0} car\xe0cters",
    "Error: Form submit field collision.": "Error: error en el camp d\u2019enviament del formulari.",
    "Error: No form element found.": "Error: no s'ha trobat l'element del formulari.",
    "Color swatch": "Mostra de color",
    "Color Picker": "Selector de colors",
    "Invalid hex color code: {0}": "Codi hex de color inv\xe0lid: {0}",
    "Invalid input": "Entrada inv\xe0lida",
    "R": "",
    "Red component": "",
    "G": "",
    "Green component": "",
    "B": "",
    "Blue component": "",
    "#": "",
    "Hex color code": "",
    "Range 0 to 255": "",
    "Turquoise": "Turquesa",
    "Green": "Verd",
    "Blue": "Blau",
    "Purple": "Violeta",
    "Navy Blue": "Blau mar\xed",
    "Dark Turquoise": "Turquesa fosc",
    "Dark Green": "Verd fosc",
    "Medium Blue": "Blau mitj\xe0",
    "Medium Purple": "Violeta mitj\xe0",
    "Midnight Blue": "Blau mitjanit",
    "Yellow": "Groc",
    "Orange": "Taronja",
    "Red": "Vermell",
    "Light Gray": "Gris clar",
    "Gray": "Gris",
    "Dark Yellow": "Groc fosc",
    "Dark Orange": "Taronja fosc",
    "Dark Red": "Vermell fosc",
    "Medium Gray": "Gris mitj\xe0",
    "Dark Gray": "Gris fosc",
    "Light Green": "Verd clar",
    "Light Yellow": "Groc clar",
    "Light Red": "Vermell clar",
    "Light Purple": "Porpra clar",
    "Light Blue": "Blau clar",
    "Dark Purple": "Porpra fosc",
    "Dark Blue": "Blau fosc",
    "Black": "Negre",
    "White": "Blanc",
    "Switch to or from fullscreen mode": "Canviar a o del mode de pantalla completa",
    "Open help dialog": "Obrir el quadre de di\xe0leg d'ajuda",
    "history": "historial",
    "styles": "estils",
    "formatting": "format",
    "alignment": "alineaci\xf3",
    "indentation": "sagnat",
    "Font": "",
    "Size": "Mida",
    "More...": "M\xe9s\u2026",
    "Select...": "Selecciona\u2026",
    "Preferences": "Par\xe0metres",
    "Yes": "S\xed",
    "No": "",
    "Keyboard Navigation": "Navegaci\xf3 per teclat",
    "Version": "Versi\xf3",
    "Code view": "Veure el codi",
    "Open popup menu for split buttons": "Obre el men\xfa emergent per als botons dividits",
    "List Properties": "Propietats de la llista",
    "List properties...": "Propietats de la llista...",
    "Start list at number": "N\xfamero on iniciar la llista",
    "Line height": "Amplada de la l\xednia",
    "Dropped file type is not supported": "",
    "Loading...": "",
    "ImageProxy HTTP error: Rejected request": "",
    "ImageProxy HTTP error: Could not find Image Proxy": "",
    "ImageProxy HTTP error: Incorrect Image Proxy URL": "",
    "ImageProxy HTTP error: Unknown ImageProxy error": ""
});