<?php

/**
 * ---------------------------------------------------------------------
 *
 * GLPI - Gestionnaire Libre de Parc Informatique
 *
 * http://glpi-project.org
 *
 * @copyright 2015-2026 Teclib' and contributors.
 * @licence   https://www.gnu.org/licenses/gpl-3.0.html
 *
 * ---------------------------------------------------------------------
 *
 * LICENSE
 *
 * This file is part of GLPI.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * ---------------------------------------------------------------------
 */

/**
 * @var \DBmysql $DB
 * @var \Migration $migration
 */

// Remove unexpected values for `pdffont` config and user preference
$migration->addPostQuery(
    $DB->buildUpdate(
        'glpi_configs',
        [
            'value' => 'dejavusans',
        ],
        [
            'name'  => 'pdffont',
            [
                'value' => ['REGEXP', '[^a-z0-9]+'],
            ],
        ]
    )
);
$migration->addPostQuery(
    $DB->buildUpdate(
        'glpi_users',
        [
            'pdffont' => null,
        ],
        [
            [
                ['pdffont' => ['REGEXP', '[^a-z0-9]+']],
                ['NOT' => ['pdffont' => null]],
            ],
        ]
    )
);
