{#
 # ---------------------------------------------------------------------
 #
 # GLPI - Gestionnaire Libre de Parc Informatique
 #
 # http://glpi-project.org
 #
 # @copyright 2015-2026 Teclib' and contributors.
 # @licence   https://www.gnu.org/licenses/gpl-3.0.html
 #
 # ---------------------------------------------------------------------
 #
 # LICENSE
 #
 # This file is part of GLPI.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # ---------------------------------------------------------------------
 #}

{% if canedit %}
    <form name="form"
          action="{{ 'Config'|itemtype_form_path }}"
          method="post"
          data-track-changes="true">
        <input type="hidden" name="_glpi_csrf_token" value="{{ csrf_token() }}" />
{% endif %}

<div class="row">
    {% block config_fields %}
    {% endblock %}
</div>

{% if canedit %}
        <div class="card-footer mx-n2 d-flex">
            <button type="submit" name="update" value="1" class="ms-auto btn btn-primary">
                <i class="ti ti-device-floppy"></i>
                <span>{{ _x('button', 'Save') }}</span>
            </button>
        </div>
    </form>
{% endif %}
