tinymce.addI18n("uz", {
    "Redo": "Bekor qilish",
    "Undo": "Orqaga qaytarish",
    "Cut": "Kesib olish",
    "Copy": "Nusxa olish",
    "Paste": "Qo\u2018yish",
    "Select all": "Barchasini belgilash",
    "New document": "Yangi hujjat",
    "Ok": "Ok",
    "Cancel": "Bekor qilish",
    "Visual aids": "Ko\u2018rgazmali o\u2018quv qurollar",
    "Bold": "Qalin",
    "Italic": "Yotiq",
    "Underline": "Tagi chizilgan",
    "Strikethrough": "O'chirilgan yozuv",
    "Superscript": "Yuqori yozuv",
    "Subscript": "Quyi yozuv",
    "Clear formatting": "Formatlashni tozalash",
    "Remove": "Olib tashlash",
    "Align left": "Chapga tekislash",
    "Align center": "Markazga tekislash",
    "Align right": "O'ngga tekislash",
    "No alignment": "",
    "Justify": "Ikki tomondan tekislash",
    "Bullet list": "Nuqtali ro\u2018yxat",
    "Numbered list": "Raqamli ro\u2018yxat",
    "Decrease indent": "Satr boshini kamaytirish",
    "Increase indent": "Satr boshini oshirish",
    "Close": "Yopish",
    "Formats": "Formatlar",
    "Your browser doesn't support direct access to the clipboard. Please use the Ctrl+X/C/V keyboard shortcuts instead.": "Sizning brauzeringiz buferga to\u2018g\u2018ridan-to\u2018g\u2018ri kirish qo\u2018llab-quvvatlamaydi. O\u2018rniga klaviaturaning Ctrl+X/C/V qisqartirishlarni foydalaning.",
    "Headings": "Sarlavhalar",
    "Heading 1": "Sarlavha 1",
    "Heading 2": "Sarlavha 2",
    "Heading 3": "Sarlavha 3",
    "Heading 4": "Sarlavha 4",
    "Heading 5": "Sarlavha 5",
    "Heading 6": "Sarlavha 6",
    "Preformatted": "",
    "Div": "Div",
    "Pre": "Pre",
    "Code": "Kod",
    "Paragraph": "Paragraf",
    "Blockquote": "Matn blok parchasi",
    "Inline": "Bir qator ketma-ketlikda",
    "Blocks": "Bloklar",
    "Paste is now in plain text mode. Contents will now be pasted as plain text until you toggle this option off.": "Joylashtirish oddiy matn rejimida amalga oshiriladi. Ushbu hususiyatni o'chirmaguningizcha, kontent oddiy matn sifatida joylashtiriladi.",
    "Fonts": "Shriftlar",
    "Font sizes": "Shrift o'lchamlari",
    "Class": "Klass",
    "Browse for an image": "Rasmni yuklash",
    "OR": "YOKI",
    "Drop an image here": "Bu erga rasmni olib o'ting",
    "Upload": "Yuklash",
    "Uploading image": "Rasm yuklanmoqda",
    "Block": "Blok",
    "Align": "Saflamoq",
    "Default": "Standart",
    "Circle": "Doira",
    "Disc": "Disk",
    "Square": "Kvadrat",
    "Lower Alpha": "Kichik lotincha",
    "Lower Greek": "Pastki yunon",
    "Lower Roman": "Kichik kirilcha",
    "Upper Alpha": "Katta lotincha",
    "Upper Roman": "Katta kirilcha",
    "Anchor...": "",
    "Anchor": "",
    "Name": "Nomi",
    "ID": "ID",
    "ID should start with a letter, followed only by letters, numbers, dashes, dots, colons or underscores.": "ID harf bilan boshlanishi kerak, faqat harflar, raqamlar, tire, nuqta, ikki nuqta yoki pastki chiziqdan iborat bo'lishi kerak.",
    "You have unsaved changes are you sure you want to navigate away?": "Sizda saqlanmagan o'zgartirishlar bor. Boshqa yerga chiqib ketish uchun ishonchingiz komilmi?",
    "Restore last draft": "Oxirgi ",
    "Special character...": "Mahsus belgi...",
    "Special Character": "Mahsus Belgi",
    "Source code": "Manba kodi",
    "Insert/Edit code sample": "Kod namunasini qo'shish / tahrirlash",
    "Language": "Til",
    "Code sample...": "Kod namunasi...",
    "Left to right": "Chapdan o'ngga",
    "Right to left": "O'ngdan chapga",
    "Title": "Nomi",
    "Fullscreen": "Butun ekran rejimi",
    "Action": "Harakat",
    "Shortcut": "Yorliq",
    "Help": "Yordam",
    "Address": "Manzil",
    "Focus to menubar": "Menubarga e'tibor qaratish",
    "Focus to toolbar": "Vositalar paneliga e'tibor qaratish",
    "Focus to element path": "Elementlar manziliga e'tibor qaratish",
    "Focus to contextual toolbar": "Kontekstli vositalar paneliga e'tibor qaratish",
    "Insert link (if link plugin activated)": "Havolani qo'shish (havola plagini o'rnatilgan bo'lsa)",
    "Save (if save plugin activated)": "Saqlash (saqlash plagini o'rnatilgan bo'lsa)",
    "Find (if searchreplace plugin activated)": "Qidirish (qidirish plagini o'rnatilgan bo'lsa)",
    "Plugins installed ({0}):": "O'rnatilgan plaginlar ({0})",
    "Premium plugins:": "Premium plaginlar:",
    "Learn more...": "Batafsil ma'lumot...",
    "You are using {0}": "Siz {0} ishlatmoqdasiz",
    "Plugins": "Plaginlar",
    "Handy Shortcuts": "Foydalanadigan yorliqlar",
    "Horizontal line": "Gorizontal chiziq",
    "Insert/edit image": "Rasmni qo'shish / tahrirlash",
    "Alternative description": "Muqobil tavsif",
    "Accessibility": "Maxsus qobiliyatlar",
    "Image is decorative": "Dekorativ rasm",
    "Source": "Manba",
    "Dimensions": "O'lchamlari",
    "Constrain proportions": "Nisbatlarni cheklash",
    "General": "Umumiy",
    "Advanced": "Ilg'or",
    "Style": "Uslub",
    "Vertical space": "Vertikal o'lchov",
    "Horizontal space": "Gorizontal o'lchov",
    "Border": "Chegara",
    "Insert image": "Rasm qo'shish",
    "Image...": "Rasm...",
    "Image list": "Rasmlar ro'yhati",
    "Resize": "O'lchamini o'zgartirish",
    "Insert date/time": "Kun / vaqtni qo'shish",
    "Date/time": "Kun/vaqt",
    "Insert/edit link": "Havola qo'shish / tahrirlash",
    "Text to display": "Ko'rsatiladigan matn",
    "Url": "URL",
    "Open link in...": "Havolani ...da ochish",
    "Current window": "Joriy oyna",
    "None": "Hech bir",
    "New window": "Yangi oyna",
    "Open link": "Havolani ochish",
    "Remove link": "Havolani olib tashlash",
    "Anchors": "Langarlar",
    "Link...": "Havola...",
    "Paste or type a link": "Havolani joylashtirish yoki kiritish",
    "The URL you entered seems to be an email address. Do you want to add the required mailto: prefix?": 'Siz kiritgan URL elektron pochta manziliga oxshayapti. "mailto:" prefiksi qo\'shilsinmi?',
    "The URL you entered seems to be an external link. Do you want to add the required http:// prefix?": 'Siz kiritgan URL tashqi havolaga oxshayapti. "http://" prefiksi qo\'shilsinmi?',
    "The URL you entered seems to be an external link. Do you want to add the required https:// prefix?": "Siz kiritgan URL tashqi havolaga o'xshaydi. Kerakli https:// prefiksini qo'shmoqchimisiz?",
    "Link list": "Havolalar ro'yhati",
    "Insert video": "Video qo'shish",
    "Insert/edit video": "Videoni qo'shish / tahrirlash",
    "Insert/edit media": "Media qo'shish / tahrirlash",
    "Alternative source": "Muqobil manba",
    "Alternative source URL": "Muqobil manba URL manzili",
    "Media poster (Image URL)": "Media poster (Rasm URL)",
    "Paste your embed code below:": "Kodni quyiga joylashtiring:",
    "Embed": "Ichiga olgan",
    "Media...": "Media...",
    "Nonbreaking space": "Buzilmas bo'sh joy",
    "Page break": "Yangi bet",
    "Paste as text": "Tekst qo'shish",
    "Preview": "Tahrirni avvaldan ko'rish",
    "Print": "Chop etish",
    "Print...": "Chop etish...",
    "Save": "Saqlash",
    "Find": "Qidirish",
    "Replace with": "bilan almashtirish",
    "Replace": "Almashtirish",
    "Replace all": "Barchasini almashtirish",
    "Previous": "Oldingi",
    "Next": "Keyingisi",
    "Find and Replace": "Qidirish va O'zgartirish",
    "Find and replace...": "Qidirish va o'zgartirish...",
    "Could not find the specified string.": "Belgilangan satr topilmadi.",
    "Match case": "O'xshashliklar",
    "Find whole words only": "Faqat to'liq so'zni topish",
    "Find in selection": "Belgilangandan topish",
    "Insert table": "Jadvalni qo'shish",
    "Table properties": "Jadval xususiyatlari",
    "Delete table": "Jadvalni o'chirib tashlash",
    "Cell": "Katak",
    "Row": "Satr",
    "Column": "Ustun",
    "Cell properties": "Katak hususiyatlari",
    "Merge cells": "Kataklarni birlashtirish",
    "Split cell": "Kataklarni bo'lish",
    "Insert row before": "Yuqorisiga satr qo'shish",
    "Insert row after": "Ketidan satr qo'shish",
    "Delete row": "Satrni olib tashlash",
    "Row properties": "Satr hususiyatlari",
    "Cut row": "Satrni kesib olish",
    "Cut column": "Ustunni kesib olish",
    "Copy row": "Satrdan nusxa ko'chirish",
    "Copy column": "Ustunni nusxalash",
    "Paste row before": "Yuqorisiga satrni joylashtirish",
    "Paste column before": "Oldinga ustun qo'yish",
    "Paste row after": "Ketidan satrni joylashtirish",
    "Paste column after": "Keyingi ustun qo'yish",
    "Insert column before": "Ustunni oldi tomoniga qo'shish",
    "Insert column after": "Ustunni ketidan qo'shish",
    "Delete column": "Ustunni olib tashlash",
    "Cols": "Ustunlar",
    "Rows": "Satrlar",
    "Width": "Kengligi",
    "Height": "Balandligi",
    "Cell spacing": "Kataklar orasi",
    "Cell padding": "Kataklar chegarasidan bo'sh joy",
    "Row clipboard actions": "Qatorning almashish buferini harakatlari",
    "Column clipboard actions": "Ustunning almashish buferini harakatlari",
    "Table styles": "Jadval stillari",
    "Cell styles": "Katak stillari",
    "Column header": "Ustun sarlavhasi",
    "Row header": "Qator sarlavhasi",
    "Table caption": "Jadval sarlavhasi",
    "Caption": "Taglavha",
    "Show caption": "Sarlavhani ko'rish",
    "Left": "Chapga",
    "Center": "Markazga",
    "Right": "O'ngga",
    "Cell type": "Katak turi",
    "Scope": "Muhit",
    "Alignment": "Tekislash",
    "Horizontal align": "Gorizontal tekislash",
    "Vertical align": "Vertikal tekislash",
    "Top": "Yuqoriga",
    "Middle": "Markaziga",
    "Bottom": "Tagiga",
    "Header cell": "Sarlavha katagi",
    "Row group": "Satrlar guruhi",
    "Column group": "Ustunlar guruhi",
    "Row type": "Satr turi",
    "Header": "Sarlavha",
    "Body": "Tanasi",
    "Footer": "Tag qismi",
    "Border color": "Chegara rangi",
    "Solid": "",
    "Dotted": "Nuqtali",
    "Dashed": "",
    "Double": "",
    "Groove": "",
    "Ridge": "",
    "Inset": "",
    "Outset": "",
    "Hidden": "Yashirin",
    "Insert template...": "Shablon joylashtirish...",
    "Templates": "Andozalar",
    "Template": "Andoza",
    "Insert Template": "Shablon Joylashtirish",
    "Text color": "Matn rangi",
    "Background color": "Orqa fon rangi",
    "Custom...": "O'zgacha...",
    "Custom color": "O'zgacha rang",
    "No color": "Rangsiz",
    "Remove color": "Rangni olib tashlash",
    "Show blocks": "Bloklarni ko'rsatish",
    "Show invisible characters": "Ko'rinmas belgilarni ko'rsatish",
    "Word count": "So'zlar soni",
    "Count": "Son",
    "Document": "Hujjat",
    "Selection": "Tanlangan",
    "Words": "So'zlar",
    "Words: {0}": "So'zlar soni: {0}",
    "{0} words": "{0} so`z",
    "File": "Fayl",
    "Edit": "Tahrirlash",
    "Insert": "Qo'shish",
    "View": "Ko'rish",
    "Format": "Shakllar",
    "Table": "Jadval",
    "Tools": "Vositalar",
    "Powered by {0}": "{0} bilan ishlaydi",
    "Rich Text Area. Press ALT-F9 for menu. Press ALT-F10 for toolbar. Press ALT-0 for help": "Murakkab matn maydoni. Menyu uchun ALT-F9 tugmalarini bosing. Vositalar paneli uchun ALT-F10 tugmasini bosing. Yordamni chaqirish uchun ALT-0-ni bosing",
    "Image title": "Rasm nomi",
    "Border width": "Chegara qalinligi",
    "Border style": "Chegara stili",
    "Error": "Xatolik",
    "Warn": "Ogohlantirish",
    "Valid": "Yaroqli",
    "To open the popup, press Shift+Enter": "Menuni ochish uchun Shift+Enter bosing",
    "Rich Text Area": "Boy Matn Maydoni",
    "Rich Text Area. Press ALT-0 for help.": "Boy Matn Maydoni. Yordam olish uchun ALT-0 bosing.",
    "System Font": "Tizim shrifti",
    "Failed to upload image: {0}": "{0}ni yuklashda xatolik yuz berdi",
    "Failed to load plugin: {0} from url {1}": "Plaginni yuklab bo\u2018lmadi: {0} {1} urldan",
    "Failed to load plugin url: {0}": "Url plaginni yuklab bo\u2018lmadi: {0}",
    "Failed to initialize plugin: {0}": "Plaginni ishga tushirib bo\u2018lmadi: {0}",
    "example": "namuna",
    "Search": "Qidirish",
    "All": "Hammasi",
    "Currency": "Valyuta",
    "Text": "Matn",
    "Quotations": "Iqtiboslar",
    "Mathematical": "Matematik",
    "Extended Latin": "Kengaytirilgan Lotin",
    "Symbols": "Belgilar",
    "Arrows": "Strelkalar",
    "User Defined": "Foydalanuvchi aniqlangan",
    "dollar sign": "dollar belgisi",
    "currency sign": "valyuta belgisi",
    "euro-currency sign": "yevro-valyuta belgisi",
    "colon sign": "colon belgisi",
    "cruzeiro sign": "cruzeiro belgisi",
    "french franc sign": "fransiya franc belgisi",
    "lira sign": "lira belgisi",
    "mill sign": "mill belgisi",
    "naira sign": "naira belgisi",
    "peseta sign": "paseta belgisi",
    "rupee sign": "rupiy belgisi",
    "won sign": "von belgisi",
    "new sheqel sign": "sheqel belgisi",
    "dong sign": "dong belgisi",
    "kip sign": "kip belgisi",
    "tugrik sign": "tugrik belgisi",
    "drachma sign": "drachma belgisi",
    "german penny symbol": "nemis pennisi belgisi",
    "peso sign": "peso belgisi",
    "guarani sign": "guarani belgisi",
    "austral sign": "austral belgisi",
    "hryvnia sign": "hryvnia belgisi",
    "cedi sign": "cedi belgisi",
    "livre tournois sign": "livre tournois belgisi",
    "spesmilo sign": "spesmilo belgisi",
    "tenge sign": "tenge belgisi",
    "indian rupee sign": "hind rupiysi belgisi",
    "turkish lira sign": "turkiya lirasi belgisi",
    "nordic mark sign": "shimoliy belgi belgisi",
    "manat sign": "manat belgisi",
    "ruble sign": "rubl belgisi",
    "yen character": "yen belgisi",
    "yuan character": "yuan belgisi",
    "yuan character, in hong kong and taiwan": "yuan belgisi, hong kong va tayvandagi",
    "yen/yuan character variant one": "yen/yuan belgisi, 1 variantda",
    "Emojis": "Emojilar",
    "Emojis...": "Emojilar...",
    "Loading emojis...": "Emojilar yuklanmoqda...",
    "Could not load emojis": "Emojilar yuklanmadi",
    "People": "Odamlar",
    "Animals and Nature": "Hayvonlar va Tabiat",
    "Food and Drink": "Ovqatlar va Ichimliklar",
    "Activity": "Harakatlar",
    "Travel and Places": "Sayohat va Joylar",
    "Objects": "Obyektlar",
    "Flags": "Bayroqlar",
    "Characters": "Belgilar",
    "Characters (no spaces)": "Belgilar (bosh joylarsiz)",
    "{0} characters": "{0} belgi",
    "Error: Form submit field collision.": "Xato: Formani yuborish maydonidagi konflikt.",
    "Error: No form element found.": "Xato: Form elementi topilmadi.",
    "Color swatch": "Rang namunasi",
    "Color Picker": "Rang tanlovchi",
    "Invalid hex color code: {0}": "Hex rang kodi noto\u2018g\u2018ri: {0}",
    "Invalid input": "Noto'g'ri kiritish",
    "R": "R",
    "Red component": "Qizil komponent",
    "G": "G",
    "Green component": "Yashil komponent",
    "B": "B",
    "Blue component": "Ko'k komponent",
    "#": "#",
    "Hex color code": "Hex rang kodi",
    "Range 0 to 255": "0 dan 255 gacha",
    "Turquoise": "",
    "Green": "Yashil",
    "Blue": "Ko'k",
    "Purple": "Siyohrang",
    "Navy Blue": "To'q ko'k rang",
    "Dark Turquoise": "",
    "Dark Green": "To'q yashil",
    "Medium Blue": "O\u02bbrtacha ko\u02bbk",
    "Medium Purple": "O\u02bbrtacha siyohrang",
    "Midnight Blue": "Yarim tun ko'k",
    "Yellow": "Sariq",
    "Orange": "Oranjiviy",
    "Red": "Qizil",
    "Light Gray": "Och kulrang",
    "Gray": "Kulrang",
    "Dark Yellow": "To'q sariq",
    "Dark Orange": "To'q oranjiviy",
    "Dark Red": "To'q qizil",
    "Medium Gray": "O'rtacha kulrang",
    "Dark Gray": "To'q kulrang",
    "Light Green": "Och yashil",
    "Light Yellow": "Och sariq",
    "Light Red": "Och qizil",
    "Light Purple": "Och siyohrang",
    "Light Blue": "Och ko'k",
    "Dark Purple": "To'q siyohrang",
    "Dark Blue": "To'q ko'k",
    "Black": "Qora",
    "White": "Oq",
    "Switch to or from fullscreen mode": "To\u02bbliq ekran rejimiga o\u02bbtish",
    "Open help dialog": "Yordamni ochish",
    "history": "tarix",
    "styles": "stillar",
    "formatting": "formatlash",
    "alignment": "tekislash",
    "indentation": "",
    "Font": "Shrift",
    "Size": "O'lcham",
    "More...": "Ko'proq...",
    "Select...": "Tanlash...",
    "Preferences": "Afzalliklar",
    "Yes": "Ha",
    "No": "Yo'q",
    "Keyboard Navigation": "Klaviatura Navigatsiyasi",
    "Version": "Versiya",
    "Code view": "Kod ko'rinishi",
    "Open popup menu for split buttons": "Ajratish tugmalari uchun menyuni ochsh",
    "List Properties": "Ro'yxat Xususiyatlari",
    "List properties...": "Ro'yxat xususiyatlari...",
    "Start list at number": "Ro'yxatni raqamdan boshlash",
    "Line height": "Chiziq balandligi",
    "Dropped file type is not supported": "Fayl turi qo'llab-quvvatlanmaydi",
    "Loading...": "Yuklanmoqda...",
    "ImageProxy HTTP error: Rejected request": "ImageProxy HTTP xato: So'rov rad etildi",
    "ImageProxy HTTP error: Could not find Image Proxy": "ImageProxy HTTP xato: Image Proxy topilmadi",
    "ImageProxy HTTP error: Incorrect Image Proxy URL": "ImageProxy HTTP xato: Image Proxy URL noto'g'ri",
    "ImageProxy HTTP error: Unknown ImageProxy error": "ImageProxy HTTP xato: Aniqlanmagan ImageProxy xato"
});